@@ -1,11 +1,28 @@
+#!/usr/bin/env python
 # coding=UTF-8
 
-from prettytable import *
-
 import sys
+import unittest
+from math import e, pi, sqrt
+
+from prettytable import (
+    ALL,
+    MARKDOWN,
+    MSWORD_FRIENDLY,
+    NONE,
+    ORGMODE,
+    PrettyTable,
+    from_csv,
+    from_db_cursor,
+    from_html,
+    from_html_one,
+    from_json,
+)
+
 py3k = sys.version_info[0] >= 3
 try:
     import sqlite3
+
     _have_sqlite = True
 except ImportError:
     _have_sqlite = False
@@ -13,20 +30,19 @@ if py3k:
     import io as StringIO
 else:
     import StringIO
-from math import pi, e, sqrt
-import unittest
 
-class BuildEquivelanceTest(unittest.TestCase):
 
-    """Make sure that building a table row-by-row and column-by-column yield the same results"""
+class BuildEquivalenceTest(unittest.TestCase):
+    """Make sure that building a table row-by-row and column-by-column yield the same
+    results"""
 
     def setUp(self):
 
         # Row by row...
         self.row = PrettyTable()
         self.row.field_names = ["City name", "Area", "Population", "Annual Rainfall"]
-        self.row.add_row(["Adelaide",1295, 1158259, 600.5])
-        self.row.add_row(["Brisbane",5905, 1857594, 1146.4])
+        self.row.add_row(["Adelaide", 1295, 1158259, 600.5])
+        self.row.add_row(["Brisbane", 5905, 1857594, 1146.4])
         self.row.add_row(["Darwin", 112, 120900, 1714.7])
         self.row.add_row(["Hobart", 1357, 205556, 619.5])
         self.row.add_row(["Sydney", 2058, 4336374, 1214.8])
@@ -35,23 +51,42 @@ class BuildEquivelanceTest(unittest.TestCase):
 
         # Column by column...
         self.col = PrettyTable()
-        self.col.add_column("City name",["Adelaide","Brisbane","Darwin","Hobart","Sydney","Melbourne","Perth"])
+        self.col.add_column(
+            "City name",
+            [
+                "Adelaide",
+                "Brisbane",
+                "Darwin",
+                "Hobart",
+                "Sydney",
+                "Melbourne",
+                "Perth",
+            ],
+        )
         self.col.add_column("Area", [1295, 5905, 112, 1357, 2058, 1566, 5386])
-        self.col.add_column("Population", [1158259, 1857594, 120900, 205556, 4336374, 3806092, 1554769])
-        self.col.add_column("Annual Rainfall",[600.5, 1146.4, 1714.7, 619.5, 1214.8, 646.9, 869.4])
+        self.col.add_column(
+            "Population", [1158259, 1857594, 120900, 205556, 4336374, 3806092, 1554769]
+        )
+        self.col.add_column(
+            "Annual Rainfall", [600.5, 1146.4, 1714.7, 619.5, 1214.8, 646.9, 869.4]
+        )
 
         # A mix of both!
         self.mix = PrettyTable()
         self.mix.field_names = ["City name", "Area"]
-        self.mix.add_row(["Adelaide",1295])
-        self.mix.add_row(["Brisbane",5905])
+        self.mix.add_row(["Adelaide", 1295])
+        self.mix.add_row(["Brisbane", 5905])
         self.mix.add_row(["Darwin", 112])
         self.mix.add_row(["Hobart", 1357])
         self.mix.add_row(["Sydney", 2058])
         self.mix.add_row(["Melbourne", 1566])
         self.mix.add_row(["Perth", 5386])
-        self.mix.add_column("Population", [1158259, 1857594, 120900, 205556, 4336374, 3806092, 1554769])
-        self.mix.add_column("Annual Rainfall",[600.5, 1146.4, 1714.7, 619.5, 1214.8, 646.9, 869.4])
+        self.mix.add_column(
+            "Population", [1158259, 1857594, 120900, 205556, 4336374, 3806092, 1554769]
+        )
+        self.mix.add_column(
+            "Annual Rainfall", [600.5, 1146.4, 1714.7, 619.5, 1214.8, 646.9, 869.4]
+        )
 
     def testRowColEquivalenceASCII(self):
 
@@ -69,7 +104,30 @@ class BuildEquivelanceTest(unittest.TestCase):
 
         self.assertEqual(self.row.get_html_string(), self.mix.get_html_string())
 
-#class FieldnamelessTableTest(unittest.TestCase):
+
+class DeleteColumnTest(unittest.TestCase):
+    def testDeleteColumn(self):
+        with_del = PrettyTable()
+        with_del.add_column("City name", ["Adelaide", "Brisbane", "Darwin"])
+        with_del.add_column("Area", [1295, 5905, 112])
+        with_del.add_column("Population", [1158259, 1857594, 120900])
+        with_del.del_column("Area")
+
+        without_row = PrettyTable()
+        without_row.add_column("City name", ["Adelaide", "Brisbane", "Darwin"])
+        without_row.add_column("Population", [1158259, 1857594, 120900])
+
+        self.assertEqual(with_del.get_string(), without_row.get_string())
+
+    def testDeleteIllegalColumnRaisesException(self):
+        table = PrettyTable()
+        table.add_column("City name", ["Adelaide", "Brisbane", "Darwin"])
+
+        with self.assertRaises(Exception):
+            table.del_column("City not-a-name")
+
+
+# class FieldnamelessTableTest(unittest.TestCase):
 #
 #    """Make sure that building and stringing a table with no fieldnames works fine"""
 #
@@ -93,6 +151,7 @@ class BuildEquivelanceTest(unittest.TestCase):
 #        self.x.field_names = ["City name", "Area", "Population", "Annual Rainfall"]
 #        self.x.get_string()
 
+
 class CityDataTest(unittest.TestCase):
 
     """Just build the Australian capital city data example table."""
@@ -100,14 +159,15 @@ class CityDataTest(unittest.TestCase):
     def setUp(self):
 
         self.x = PrettyTable(["City name", "Area", "Population", "Annual Rainfall"])
-        self.x.add_row(["Adelaide",1295, 1158259, 600.5])
-        self.x.add_row(["Brisbane",5905, 1857594, 1146.4])
+        self.x.add_row(["Adelaide", 1295, 1158259, 600.5])
+        self.x.add_row(["Brisbane", 5905, 1857594, 1146.4])
         self.x.add_row(["Darwin", 112, 120900, 1714.7])
         self.x.add_row(["Hobart", 1357, 205556, 619.5])
         self.x.add_row(["Sydney", 2058, 4336374, 1214.8])
         self.x.add_row(["Melbourne", 1566, 3806092, 646.9])
         self.x.add_row(["Perth", 5386, 1554769, 869.4])
 
+
 class OptionOverrideTests(CityDataTest):
 
     """Make sure all options are properly overwritten by get_string."""
@@ -115,23 +175,24 @@ class OptionOverrideTests(CityDataTest):
     def testBorder(self):
         default = self.x.get_string()
         override = self.x.get_string(border=False)
-        self.assertTrue(default != override)
+        self.assertNotEqual(default, override)
 
     def testHeader(self):
         default = self.x.get_string()
         override = self.x.get_string(header=False)
-        self.assertTrue(default != override)
+        self.assertNotEqual(default, override)
 
     def testHrulesAll(self):
         default = self.x.get_string()
         override = self.x.get_string(hrules=ALL)
-        self.assertTrue(default != override)
+        self.assertNotEqual(default, override)
 
     def testHrulesNone(self):
 
         default = self.x.get_string()
         override = self.x.get_string(hrules=NONE)
-        self.assertTrue(default != override)
+        self.assertNotEqual(default, override)
+
 
 class OptionAttributeTests(CityDataTest):
 
@@ -159,7 +220,7 @@ class OptionAttributeTests(CityDataTest):
         self.x.horizontal_char = "~"
         self.x.junction_char = "*"
         self.x.format = True
-        self.x.attributes = {"class" : "prettytable"}
+        self.x.attributes = {"class": "prettytable"}
         assert self.x.get_string() == self.x[:].get_string()
 
     def testSetForOneColumn(self):
@@ -169,6 +230,7 @@ class OptionAttributeTests(CityDataTest):
         self.x.float_format["Area"] = "2.2"
         assert self.x.get_string() == self.x[:].get_string()
 
+
 class BasicTests(CityDataTest):
 
     """Some very basic tests."""
@@ -179,7 +241,7 @@ class BasicTests(CityDataTest):
 
         string = self.x.get_string()
         lines = string.split("\n")
-        self.assertTrue("" not in lines)
+        self.assertNotIn("", lines)
 
     def testAllLengthsEqual(self):
 
@@ -189,7 +251,17 @@ class BasicTests(CityDataTest):
         lines = string.split("\n")
         lengths = [len(line) for line in lines]
         lengths = set(lengths)
-        self.assertEqual(len(lengths),1)
+        self.assertEqual(len(lengths), 1)
+
+
+class TitleBasicTests(BasicTests):
+
+    """Run the basic tests with a title"""
+
+    def setUp(self):
+        BasicTests.setUp(self)
+        self.x.title = "My table"
+
 
 class NoBorderBasicTests(BasicTests):
 
@@ -199,6 +271,7 @@ class NoBorderBasicTests(BasicTests):
         BasicTests.setUp(self)
         self.x.border = False
 
+
 class NoHeaderBasicTests(BasicTests):
 
     """Run the basic tests with header = False"""
@@ -207,6 +280,7 @@ class NoHeaderBasicTests(BasicTests):
         BasicTests.setUp(self)
         self.x.header = False
 
+
 class HrulesNoneBasicTests(BasicTests):
 
     """Run the basic tests with hrules = NONE"""
@@ -215,6 +289,7 @@ class HrulesNoneBasicTests(BasicTests):
         BasicTests.setUp(self)
         self.x.hrules = NONE
 
+
 class HrulesAllBasicTests(BasicTests):
 
     """Run the basic tests with hrules = ALL"""
@@ -223,6 +298,32 @@ class HrulesAllBasicTests(BasicTests):
         BasicTests.setUp(self)
         self.x.hrules = ALL
 
+
+class EmptyTableTests(CityDataTest):
+
+    """Make sure the print_empty option works"""
+
+    def setUp(self):
+        CityDataTest.setUp(self)
+        self.y = PrettyTable()
+        self.y.field_names = ["City name", "Area", "Population", "Annual Rainfall"]
+
+    def testPrintEmptyTrue(self):
+        assert self.y.get_string(print_empty=True) != ""
+        assert self.x.get_string(print_empty=True) != self.y.get_string(
+            print_empty=True
+        )
+
+    def testPrintEmptyFalse(self):
+        assert self.y.get_string(print_empty=False) == ""
+        assert self.y.get_string(print_empty=False) != self.x.get_string(
+            print_empty=False
+        )
+
+    def testInteractionWithBorder(self):
+        assert self.y.get_string(border=False, print_empty=True) == ""
+
+
 class PresetBasicTests(BasicTests):
 
     """Run the basic tests after using set_style"""
@@ -231,8 +332,8 @@ class PresetBasicTests(BasicTests):
         BasicTests.setUp(self)
         self.x.set_style(MSWORD_FRIENDLY)
 
-class SlicingTests(CityDataTest):
 
+class SlicingTests(CityDataTest):
     def setUp(self):
         CityDataTest.setUp(self)
 
@@ -258,8 +359,8 @@ class SlicingTests(CityDataTest):
         assert "Melbourne" in string
         assert "Perth" in string
 
-class SortingTests(CityDataTest):
 
+class SortingTests(CityDataTest):
     def setUp(self):
         CityDataTest.setUp(self)
 
@@ -278,7 +379,7 @@ class SortingTests(CityDataTest):
             forward = self.x.get_string()
             self.x.reversesort = True
             backward = self.x.get_string()
-            forward_lines = forward.split("\n")[2:] # Discard header lines
+            forward_lines = forward.split("\n")[2:]  # Discard header lines
             backward_lines = backward.split("\n")[2:]
             backward_lines.reverse()
             assert forward_lines == backward_lines
@@ -288,9 +389,12 @@ class SortingTests(CityDataTest):
         def key(vals):
             vals[0] = len(vals[0])
             return vals
+
         self.x.sortby = "City name"
         self.x.sort_key = key
-        assert self.x.get_string().strip() == """+-----------+------+------------+-----------------+
+        assert (
+            self.x.get_string().strip()
+            == """+-----------+------+------------+-----------------+
 | City name | Area | Population | Annual Rainfall |
 +-----------+------+------------+-----------------+
 |   Perth   | 5386 |  1554769   |      869.4      |
@@ -302,6 +406,20 @@ class SortingTests(CityDataTest):
 | Melbourne | 1566 |  3806092   |      646.9      |
 +-----------+------+------------+-----------------+
 """.strip()
+        )
+
+    def testSortSlice(self):
+        """Make sure sorting and slicing interact in the expected way"""
+        x = PrettyTable(["Foo"])
+        for i in range(20, 0, -1):
+            x.add_row([i])
+        newstyle = x.get_string(sortby="Foo", end=10)
+        assert "10" in newstyle
+        assert "20" not in newstyle
+        oldstyle = x.get_string(sortby="Foo", end=10, oldsortslice=True)
+        assert "10" not in oldstyle
+        assert "20" in oldstyle
+
 
 class IntegerFormatBasicTests(BasicTests):
 
@@ -311,6 +429,7 @@ class IntegerFormatBasicTests(BasicTests):
         BasicTests.setUp(self)
         self.x.int_format = "04"
 
+
 class FloatFormatBasicTests(BasicTests):
 
     """Run the basic tests after setting a float format string"""
@@ -319,13 +438,13 @@ class FloatFormatBasicTests(BasicTests):
         BasicTests.setUp(self)
         self.x.float_format = "6.2f"
 
-class FloatFormatTests(unittest.TestCase):
 
+class FloatFormatTests(unittest.TestCase):
     def setUp(self):
         self.x = PrettyTable(["Constant", "Value"])
-        self.x.add_row(["Pi", pi]) 
-        self.x.add_row(["e", e]) 
-        self.x.add_row(["sqrt(2)", sqrt(2)]) 
+        self.x.add_row(["Pi", pi])
+        self.x.add_row(["e", e])
+        self.x.add_row(["sqrt(2)", sqrt(2)])
 
     def testNoDecimals(self):
         self.x.float_format = ".0f"
@@ -337,9 +456,9 @@ class FloatFormatTests(unittest.TestCase):
         string = self.x.get_string()
         assert "3.14159" in string
         assert "3.141592" not in string
-        assert "2.71828" in string 
-        assert "2.718281" not in string 
-        assert "2.718282" not in string 
+        assert "2.71828" in string
+        assert "2.718281" not in string
+        assert "2.718282" not in string
         assert "1.41421" in string
         assert "1.414213" not in string
 
@@ -350,13 +469,16 @@ class FloatFormatTests(unittest.TestCase):
         assert "002.72" in string
         assert "001.41" in string
 
+
 class BreakLineTests(unittest.TestCase):
     def testAsciiBreakLine(self):
-        t = PrettyTable(['Field 1', 'Field 2'])
-        t.add_row(['value 1', 'value2\nsecond line'])
-        t.add_row(['value 3', 'value4'])
+        t = PrettyTable(["Field 1", "Field 2"])
+        t.add_row(["value 1", "value2\nsecond line"])
+        t.add_row(["value 3", "value4"])
         result = t.get_string(hrules=ALL)
-        assert result.strip() == """
+        assert (
+            result.strip()
+            == """
 +---------+-------------+
 | Field 1 |   Field 2   |
 +---------+-------------+
@@ -366,12 +488,15 @@ class BreakLineTests(unittest.TestCase):
 | value 3 |    value4   |
 +---------+-------------+
 """.strip()
+        )
 
-        t = PrettyTable(['Field 1', 'Field 2'])
-        t.add_row(['value 1', 'value2\nsecond line'])
-        t.add_row(['value 3\n\nother line', 'value4\n\n\nvalue5'])
+        t = PrettyTable(["Field 1", "Field 2"])
+        t.add_row(["value 1", "value2\nsecond line"])
+        t.add_row(["value 3\n\nother line", "value4\n\n\nvalue5"])
         result = t.get_string(hrules=ALL)
-        assert result.strip() == """
+        assert (
+            result.strip()
+            == """
 +------------+-------------+
 |  Field 1   |   Field 2   |
 +------------+-------------+
@@ -384,12 +509,15 @@ class BreakLineTests(unittest.TestCase):
 |            |    value5   |
 +------------+-------------+
 """.strip()
+        )
 
-        t = PrettyTable(['Field 1', 'Field 2'])
-        t.add_row(['value 1', 'value2\nsecond line'])
-        t.add_row(['value 3\n\nother line', 'value4\n\n\nvalue5'])
+        t = PrettyTable(["Field 1", "Field 2"])
+        t.add_row(["value 1", "value2\nsecond line"])
+        t.add_row(["value 3\n\nother line", "value4\n\n\nvalue5"])
         result = t.get_string()
-        assert result.strip() == """
+        assert (
+            result.strip()
+            == """
 +------------+-------------+
 |  Field 1   |   Field 2   |
 +------------+-------------+
@@ -401,13 +529,16 @@ class BreakLineTests(unittest.TestCase):
 |            |    value5   |
 +------------+-------------+
 """.strip()
+        )
 
     def testHtmlBreakLine(self):
-        t = PrettyTable(['Field 1', 'Field 2'])
-        t.add_row(['value 1', 'value2\nsecond line'])
-        t.add_row(['value 3', 'value4'])
+        t = PrettyTable(["Field 1", "Field 2"])
+        t.add_row(["value 1", "value2\nsecond line"])
+        t.add_row(["value 3", "value4"])
         result = t.get_html_string(hrules=ALL)
-        assert result.strip() == """
+        assert (
+            result.strip()
+            == """
 <table>
     <tr>
         <th>Field 1</th>
@@ -415,7 +546,7 @@ class BreakLineTests(unittest.TestCase):
     </tr>
     <tr>
         <td>value 1</td>
-        <td>value2<br />second line</td>
+        <td>value2<br>second line</td>
     </tr>
     <tr>
         <td>value 3</td>
@@ -423,16 +554,53 @@ class BreakLineTests(unittest.TestCase):
     </tr>
 </table>
 """.strip()
+        )
+
+
+class JSONOutputTests(unittest.TestCase):
+    def testJSONOutput(self):
+        t = PrettyTable(["Field 1", "Field 2", "Field 3"])
+        t.add_row(["value 1", "value2", "value3"])
+        t.add_row(["value 4", "value5", "value6"])
+        t.add_row(["value 7", "value8", "value9"])
+        result = t.get_json_string()
+        assert (
+            result.strip()
+            == """[
+    [
+        "Field 1",
+        "Field 2",
+        "Field 3"
+    ],
+    {
+        "Field 1": "value 1",
+        "Field 2": "value2",
+        "Field 3": "value3"
+    },
+    {
+        "Field 1": "value 4",
+        "Field 2": "value5",
+        "Field 3": "value6"
+    },
+    {
+        "Field 1": "value 7",
+        "Field 2": "value8",
+        "Field 3": "value9"
+    }
+]""".strip()
+        )
+
 
 class HtmlOutputTests(unittest.TestCase):
-
     def testHtmlOutput(self):
-        t = PrettyTable(['Field 1', 'Field 2', 'Field 3'])
-        t.add_row(['value 1', 'value2', 'value3'])
-        t.add_row(['value 4', 'value5', 'value6'])
-        t.add_row(['value 7', 'value8', 'value9'])
+        t = PrettyTable(["Field 1", "Field 2", "Field 3"])
+        t.add_row(["value 1", "value2", "value3"])
+        t.add_row(["value 4", "value5", "value6"])
+        t.add_row(["value 7", "value8", "value9"])
         result = t.get_html_string()
-        assert result.strip() == """
+        assert (
+            result.strip()
+            == """
 <table>
     <tr>
         <th>Field 1</th>
@@ -456,14 +624,17 @@ class HtmlOutputTests(unittest.TestCase):
     </tr>
 </table>
 """.strip()
+        )
 
     def testHtmlOutputFormated(self):
-        t = PrettyTable(['Field 1', 'Field 2', 'Field 3'])
-        t.add_row(['value 1', 'value2', 'value3'])
-        t.add_row(['value 4', 'value5', 'value6'])
-        t.add_row(['value 7', 'value8', 'value9'])
+        t = PrettyTable(["Field 1", "Field 2", "Field 3"])
+        t.add_row(["value 1", "value2", "value3"])
+        t.add_row(["value 4", "value5", "value6"])
+        t.add_row(["value 7", "value8", "value9"])
         result = t.get_html_string(format=True)
-        assert result.strip() == """
+        assert (
+            result.strip()
+            == """
 <table frame="box" rules="cols">
     <tr>
         <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 1</th>
@@ -486,10 +657,53 @@ class HtmlOutputTests(unittest.TestCase):
         <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value9</td>
     </tr>
 </table>
+""".strip()  # noqa: E501
+        )
+
+
+class MarkdownStyleTest(BasicTests):
+    def testMarkdownStyle(self):
+        t = PrettyTable(["Field 1", "Field 2", "Field 3"])
+        t.add_row(["value 1", "value2", "value3"])
+        t.add_row(["value 4", "value5", "value6"])
+        t.add_row(["value 7", "value8", "value9"])
+        t.set_style(MARKDOWN)
+        result = t.get_string()
+        assert (
+            result.strip()
+            == """
+| Field 1 | Field 2 | Field 3 |
+|---------|---------|---------|
+| value 1 |  value2 |  value3 |
+| value 4 |  value5 |  value6 |
+| value 7 |  value8 |  value9 |
 """.strip()
+        )
+
+
+class OrgmodeStyleTest(BasicTests):
+    def testOrgmodeStyle(self):
+        t = PrettyTable(["Field 1", "Field 2", "Field 3"])
+        t.add_row(["value 1", "value2", "value3"])
+        t.add_row(["value 4", "value5", "value6"])
+        t.add_row(["value 7", "value8", "value9"])
+        t.set_style(ORGMODE)
+        result = t.get_string()
+        assert (
+            result.strip()
+            == """
+|---------+---------+---------|
+| Field 1 | Field 2 | Field 3 |
+|---------+---------+---------|
+| value 1 |  value2 |  value3 |
+| value 4 |  value5 |  value6 |
+| value 7 |  value8 |  value9 |
+|---------+---------+---------|
+""".strip()
+        )
+
 
 class CsvConstructorTest(BasicTests):
-
     def setUp(self):
 
         csv_string = """City name, Area , Population , Annual Rainfall
@@ -503,29 +717,77 @@ class CsvConstructorTest(BasicTests):
         csv_fp = StringIO.StringIO(csv_string)
         self.x = from_csv(csv_fp)
 
-if _have_sqlite:
-    class DatabaseConstructorTest(BasicTests):
 
+class CsvOutputTests(unittest.TestCase):
+    def testCsvOutput(self):
+        t = PrettyTable(["Field 1", "Field 2", "Field 3"])
+        t.add_row(["value 1", "value2", "value3"])
+        t.add_row(["value 4", "value5", "value6"])
+        t.add_row(["value 7", "value8", "value9"])
+        self.assertEqual(
+            t.get_csv_string(delimiter="\t", header=False),
+            "value 1\tvalue2\tvalue3\r\n"
+            "value 4\tvalue5\tvalue6\r\n"
+            "value 7\tvalue8\tvalue9\r\n",
+        )
+        self.assertEqual(
+            t.get_csv_string(),
+            "Field 1,Field 2,Field 3\r\n"
+            "value 1,value2,value3\r\n"
+            "value 4,value5,value6\r\n"
+            "value 7,value8,value9\r\n",
+        )
+
+
+if _have_sqlite:
+
+    class DatabaseConstructorTest(BasicTests):
         def setUp(self):
             self.conn = sqlite3.connect(":memory:")
             self.cur = self.conn.cursor()
-            self.cur.execute("CREATE TABLE cities (name TEXT, area INTEGER, population INTEGER, rainfall REAL)")
-            self.cur.execute("INSERT INTO cities VALUES (\"Adelaide\", 1295, 1158259, 600.5)")
-            self.cur.execute("INSERT INTO cities VALUES (\"Brisbane\", 5905, 1857594, 1146.4)")
-            self.cur.execute("INSERT INTO cities VALUES (\"Darwin\", 112, 120900, 1714.7)")
-            self.cur.execute("INSERT INTO cities VALUES (\"Hobart\", 1357, 205556, 619.5)")
-            self.cur.execute("INSERT INTO cities VALUES (\"Sydney\", 2058, 4336374, 1214.8)")
-            self.cur.execute("INSERT INTO cities VALUES (\"Melbourne\", 1566, 3806092, 646.9)")
-            self.cur.execute("INSERT INTO cities VALUES (\"Perth\", 5386, 1554769, 869.4)")
+            self.cur.execute(
+                "CREATE TABLE cities "
+                "(name TEXT, area INTEGER, population INTEGER, rainfall REAL)"
+            )
+            self.cur.execute(
+                'INSERT INTO cities VALUES ("Adelaide", 1295, 1158259, 600.5)'
+            )
+            self.cur.execute(
+                'INSERT INTO cities VALUES ("Brisbane", 5905, 1857594, 1146.4)'
+            )
+            self.cur.execute(
+                'INSERT INTO cities VALUES ("Darwin", 112, 120900, 1714.7)'
+            )
+            self.cur.execute(
+                'INSERT INTO cities VALUES ("Hobart", 1357, 205556, 619.5)'
+            )
+            self.cur.execute(
+                'INSERT INTO cities VALUES ("Sydney", 2058, 4336374, 1214.8)'
+            )
+            self.cur.execute(
+                'INSERT INTO cities VALUES ("Melbourne", 1566, 3806092, 646.9)'
+            )
+            self.cur.execute(
+                'INSERT INTO cities VALUES ("Perth", 5386, 1554769, 869.4)'
+            )
             self.cur.execute("SELECT * FROM cities")
             self.x = from_db_cursor(self.cur)
 
-        def testNonSelectCurosr(self):
-            self.cur.execute("INSERT INTO cities VALUES (\"Adelaide\", 1295, 1158259, 600.5)")
+        def testNonSelectCursor(self):
+            self.cur.execute(
+                'INSERT INTO cities VALUES ("Adelaide", 1295, 1158259, 600.5)'
+            )
             assert from_db_cursor(self.cur) is None
 
-class HtmlConstructorTest(CityDataTest):
 
+class JSONConstructorTest(CityDataTest):
+    def testJSONAndBack(self):
+        json_string = self.x.get_json_string()
+        new_table = from_json(json_string)
+        assert new_table.get_string() == self.x.get_string()
+
+
+class HtmlConstructorTest(CityDataTest):
     def testHtmlAndBack(self):
         html_string = self.x.get_html_string()
         new_table = from_html(html_string)[0]
@@ -541,14 +803,14 @@ class HtmlConstructorTest(CityDataTest):
         html_string += self.x.get_html_string()
         self.assertRaises(Exception, from_html_one, html_string)
 
-class PrintEnglishTest(CityDataTest):
 
+class PrintEnglishTest(CityDataTest):
     def testPrint(self):
         print()
         print(self.x)
 
-class PrintJapanestTest(unittest.TestCase):
 
+class PrintJapaneseTest(unittest.TestCase):
     def setUp(self):
 
         self.x = PrettyTable(["Kanji", "Hiragana", "English"])
@@ -565,5 +827,33 @@ class PrintJapanestTest(unittest.TestCase):
         print()
         print(self.x)
 
+
+class PrintEmojiTest(unittest.TestCase):
+    def setUp(self):
+        thunder1 = [
+            '\033[38;5;226m _`/""\033[38;5;250m.-.    \033[0m',
+            "\033[38;5;226m  ,\\_\033[38;5;250m(   ).  \033[0m",
+            "\033[38;5;226m   /\033[38;5;250m(___(__) \033[0m",
+            "\033[38;5;228;5m    ⚡\033[38;5;111;25mʻ ʻ\033[38;5;228;5m"
+            "⚡\033[38;5;111;25mʻ ʻ \033[0m",
+            "\033[38;5;111m    ʻ ʻ ʻ ʻ  \033[0m",
+        ]
+        thunder2 = [
+            "\033[38;5;240;1m     .-.     \033[0m",
+            "\033[38;5;240;1m    (   ).   \033[0m",
+            "\033[38;5;240;1m   (___(__)  \033[0m",
+            "\033[38;5;21;1m  ‚ʻ\033[38;5;228;5m⚡\033[38;5;21;25mʻ‚\033[38;5;228;5m"
+            "⚡\033[38;5;21;25m‚ʻ   \033[0m",
+            "\033[38;5;21;1m  ‚ʻ‚ʻ\033[38;5;228;5m⚡\033[38;5;21;25mʻ‚ʻ   \033[0m",
+        ]
+        self.x = PrettyTable(["Thunderbolt", "Lightning"])
+        for i in range(len(thunder1)):
+            self.x.add_row([thunder1[i], thunder2[i]])
+
+    def testPrint(self):
+        print()
+        print(self.x)
+
+
 if __name__ == "__main__":
     unittest.main()
